@@ -46,46 +46,47 @@ PRp.toString = function() {
 
 var emptyPrintResult = new PrintResult("");
 
-function Printer(originalOptions) {
+function Printer(config) {
     assert.ok(this instanceof Printer);
 
-    var explicitTabWidth = originalOptions && originalOptions.tabWidth;
-    var options = normalizeOptions(originalOptions);
-    assert.notStrictEqual(options, originalOptions);
+    const explicitTabWidth = config && config.tabWidth;
+    config = normalizeOptions(config);
 
     // It's common for client code to pass the same options into both
     // recast.parse and recast.print, but the Printer doesn't need (and
-    // can be confused by) options.sourceFileName, so we null it out.
-    options.sourceFileName = null;
+    // can be confused by) config.sourceFileName, so we null it out.
+    config.sourceFileName = null;
 
-    function printWithComments(path) {
-        assert.ok(path instanceof FastPath);
-        return printComments(path, print);
+    // Non-destructively modifies options with overrides, and returns a
+    // new print function that uses the modified options.
+    function makePrintFunctionWith(options, overrides) {
+        options = Object.assign({}, options, overrides);
+        return function (path) {
+            return print(path, options);
+        };
     }
 
-    function print(path, includeComments) {
-        if (includeComments)
-            return printWithComments(path);
-
+    function print(path, options) {
         assert.ok(path instanceof FastPath);
+        options = options || {};
 
-        if (!explicitTabWidth) {
-            var oldTabWidth = options.tabWidth;
-            var loc = path.getNode().loc;
+        if (options.includeComments) {
+            return printComments(path, makePrintFunctionWith(options, {
+                includeComments: false
+            }));
+        }
+
+        const oldTabWidth = config.tabWidth;
+
+        if (! explicitTabWidth) {
+            const loc = path.getNode().loc;
             if (loc && loc.lines && loc.lines.guessTabWidth) {
-                options.tabWidth = loc.lines.guessTabWidth();
-                var lines = maybeReprint(path);
-                options.tabWidth = oldTabWidth;
-                return lines;
+                config.tabWidth = loc.lines.guessTabWidth();
             }
         }
 
-        return maybeReprint(path);
-    }
-
-    function maybeReprint(path) {
-        var reprinter = getReprinter(path);
-        if (reprinter) {
+        const reprinter = getReprinter(path);
+        const lines = reprinter
             // Since the print function that we pass to the reprinter will
             // be used to print "new" nodes, it's tempting to think we
             // should pass printRootGenerically instead of print, to avoid
@@ -94,22 +95,20 @@ function Printer(originalOptions) {
             // moved from elsewhere in the AST. The print function is the
             // right choice because it gives us the opportunity to reprint
             // such nodes using their original source.
-            return maybeAddParens(path, reprinter(print));
-        }
-        return printRootGenerically(path);
-    }
+            ? reprinter(print)
+            : genericPrint(
+                path,
+                config,
+                options,
+                makePrintFunctionWith(options, {
+                    includeComments: true,
+                    avoidRootParens: false
+                })
+            );
 
-    // Print the root node generically, but then resume reprinting its
-    // children non-generically.
-    function printRootGenerically(path, includeComments) {
-        return includeComments
-            ? printComments(path, printRootGenerically)
-            : genericPrint(path, options, printWithComments);
-    }
+        config.tabWidth = oldTabWidth;
 
-    // Print the entire AST generically.
-    function printGenerically(path) {
-        return genericPrint(path, options, printGenerically);
+        return lines;
     }
 
     this.print = function(ast) {
@@ -117,15 +116,18 @@ function Printer(originalOptions) {
             return emptyPrintResult;
         }
 
-        var lines = print(FastPath.from(ast), true);
+        const lines = print(FastPath.from(ast), {
+            includeComments: true,
+            avoidRootParens: false
+        });
 
         return new PrintResult(
-            lines.toString(options),
+            lines.toString(config),
             util.composeSourceMaps(
-                options.inputSourceMap,
+                config.inputSourceMap,
                 lines.getSourceMap(
-                    options.sourceMapName,
-                    options.sourceRoot
+                    config.sourceMapName,
+                    config.sourceRoot
                 )
             )
         );
@@ -136,77 +138,74 @@ function Printer(originalOptions) {
             return emptyPrintResult;
         }
 
-        var path = FastPath.from(ast);
-        var oldReuseWhitespace = options.reuseWhitespace;
+        // Print the entire AST generically.
+        function printGenerically(path) {
+            return printComments(path, function (path) {
+                return genericPrint(path, config, {
+                    includeComments: true,
+                    avoidRootParens: false
+                }, printGenerically);
+            });
+        }
+
+        const path = FastPath.from(ast);
+        const oldReuseWhitespace = config.reuseWhitespace;
 
         // Do not reuse whitespace (or anything else, for that matter)
         // when printing generically.
-        options.reuseWhitespace = false;
+        config.reuseWhitespace = false;
 
         // TODO Allow printing of comments?
-        var pr = new PrintResult(printGenerically(path).toString(options));
-        options.reuseWhitespace = oldReuseWhitespace;
+        const pr = new PrintResult(printGenerically(path).toString(config));
+        config.reuseWhitespace = oldReuseWhitespace;
         return pr;
     };
 }
 
 exports.Printer = Printer;
 
-function maybeAddParens(path, lines) {
-    return path.needsParens() ? concat(["(", lines, ")"]) : lines;
-}
-
-function genericPrint(path, options, printPath) {
+function genericPrint(path, config, options, printPath) {
     assert.ok(path instanceof FastPath);
 
-    var node = path.getValue();
-    var parts = [];
-    var needsParens = false;
-    var linesWithoutParens =
-        genericPrintNoParens(path, options, printPath);
+    const node = path.getValue();
+    const parts = [];
+    const linesWithoutParens =
+        genericPrintNoParens(path, config, printPath);
 
     if (! node || linesWithoutParens.isEmpty()) {
         return linesWithoutParens;
     }
 
-    if (node.decorators &&
-        node.decorators.length > 0 &&
-        // If the parent node is an export declaration, it will be
-        // responsible for printing node.decorators.
-        ! util.getParentExportDeclaration(path)) {
+    let shouldAddParens = false;
+    const decoratorsLines = printDecorators(path, printPath);
 
-        path.each(function(decoratorPath) {
-            parts.push(printPath(decoratorPath), "\n");
-        }, "decorators");
-
-    } else if (util.isExportDeclaration(node) &&
-               node.declaration &&
-               node.declaration.decorators) {
-        // Export declarations are responsible for printing any decorators
-        // that logically apply to node.declaration.
-        path.each(function(decoratorPath) {
-            parts.push(printPath(decoratorPath), "\n");
-        }, "declaration", "decorators");
-
-    } else {
+    if (decoratorsLines.isEmpty()) {
         // Nodes with decorators can't have parentheses, so we can avoid
         // computing path.needsParens() except in this case.
-        needsParens = path.needsParens();
+        if (! options.avoidRootParens) {
+            shouldAddParens = path.needsParens();
+        }
+    } else {
+        parts.push(decoratorsLines);
     }
 
-    if (needsParens) {
+    if (shouldAddParens) {
         parts.unshift("(");
     }
 
     parts.push(linesWithoutParens);
 
-    if (needsParens) {
+    if (shouldAddParens) {
         parts.push(")");
     }
 
     return concat(parts);
 }
 
+// Note that the `options` parameter of this function is what other
+// functions in this file call the `config` object (that is, the
+// configuration object originally passed into the Printer constructor).
+// Its properties are documented in lib/options.js.
 function genericPrintNoParens(path, options, print) {
     var n = path.getValue();
 
@@ -701,6 +700,10 @@ function genericPrintNoParens(path, options, print) {
             parts[parts.length - 1] = " " + rightBrace;
         }
 
+        if (n.typeAnnotation) {
+            parts.push(path.call(print, "typeAnnotation"));
+        }
+
         return concat(parts);
 
     case "PropertyPattern":
@@ -872,9 +875,9 @@ function genericPrintNoParens(path, options, print) {
 
     case "ConditionalExpression":
         return concat([
-            "(", path.call(print, "test"),
+            path.call(print, "test"),
             " ? ", path.call(print, "consequent"),
-            " : ", path.call(print, "alternate"), ")"
+            " : ", path.call(print, "alternate")
         ]);
 
     case "NewExpression":
@@ -1833,6 +1836,9 @@ function genericPrintNoParens(path, options, print) {
     case "TSUndefinedKeyword":
         return fromString("undefined", options);
 
+    case "TSUnknownKeyword":
+        return fromString("unknown", options);
+
     case "TSNeverKeyword":
         return fromString("never", options);
 
@@ -1916,6 +1922,19 @@ function genericPrintNoParens(path, options, print) {
             "]"
         ]);
 
+    case "TSRestType":
+        return concat([
+            "...",
+            path.call(print, "typeAnnotation"),
+            "[]"
+        ]);
+
+    case "TSOptionalType":
+        return concat([
+            path.call(print, "typeAnnotation"),
+            "?"
+        ]);
+
     case "TSIndexedAccessType":
         return concat([
             path.call(print, "objectType"),
@@ -2354,6 +2373,33 @@ function genericPrintNoParens(path, options, print) {
     return p;
 }
 
+function printDecorators(path, printPath) {
+    const parts = [];
+    const node = path.getValue();
+
+    if (node.decorators &&
+        node.decorators.length > 0 &&
+        // If the parent node is an export declaration, it will be
+        // responsible for printing node.decorators.
+        ! util.getParentExportDeclaration(path)) {
+
+        path.each(function(decoratorPath) {
+            parts.push(printPath(decoratorPath), "\n");
+        }, "decorators");
+
+    } else if (util.isExportDeclaration(node) &&
+               node.declaration &&
+               node.declaration.decorators) {
+        // Export declarations are responsible for printing any decorators
+        // that logically apply to node.declaration.
+        path.each(function(decoratorPath) {
+            parts.push(printPath(decoratorPath), "\n");
+        }, "declaration", "decorators");
+    }
+
+    return concat(parts);
+}
+
 function printStatementSequence(path, options, print) {
     var inClassBody =
         namedTypes.ClassBody &&
@@ -2375,7 +2421,8 @@ function printStatementSequence(path, options, print) {
 
         // Skip printing EmptyStatement nodes to avoid leaving stray
         // semicolons lying around.
-        if (stmt.type === "EmptyStatement") {
+        if (stmt.type === "EmptyStatement" &&
+            ! (stmt.comments && stmt.comments.length > 0)) {
             return;
         }
 
@@ -2654,12 +2701,15 @@ function printExportDeclaration(path, options, print) {
     if (decl.declaration) {
         parts.push(path.call(print, "declaration"));
 
-    } else if (decl.specifiers &&
-               decl.specifiers.length > 0) {
+    } else if (decl.specifiers) {
 
         if (decl.specifiers.length === 1 &&
             decl.specifiers[0].type === "ExportBatchSpecifier") {
             parts.push("*");
+
+        } else if (decl.specifiers.length === 0) {
+            parts.push("{}");
+
         } else if (decl.specifiers[0].type === 'ExportDefaultSpecifier') {
             const unbracedSpecifiers = [];
             const bracedSpecifiers = [];
@@ -2703,6 +2753,7 @@ function printExportDeclaration(path, options, print) {
                     parts.push("{", lines, "}");
                 }
             }
+
         } else {
             parts.push(
                 shouldPrintSpaces ? "{ " : "{",
