@@ -16,8 +16,9 @@ exports.parse = function (source, options) {
     locations: true,
     comment: true,
     onComment: comments,
+    range: getOption(options, "range", false),
     tolerant: getOption(options, "tolerant", true),
-    tokens: getOption(options, "tokens", true)
+    tokens: true
   });
 
   if (! Array.isArray(ast.comments)) {
