@@ -292,12 +292,20 @@ describe("TypeScript", function() {
       '  public static async *[name]<T>(arg: U): V;',
       '}'
     ]);
+
+    check([
+      'function myFunction(',
+      '  {',
+      '    param1',
+      '  }: Params',
+      ') {}'
+    ]);
   });
 });
 
 testReprinting(
   "data/babel-parser/test/fixtures/typescript/**/input.js",
-  "Reprinting Babylon TypeScript test fixtures"
+  "Reprinting @babel/parser TypeScript test fixtures"
 );
 
 testReprinting(
@@ -311,7 +319,8 @@ function testReprinting(pattern, description) {
       cwd: __dirname
     }).forEach(file => it(file, function () {
       if (file.indexOf("/tsx/") >= 0 ||
-          file.endsWith("stitching/errors.ts")) {
+          file.endsWith("stitching/errors.ts") ||
+          file.endsWith("decorators/type-arguments-invalid/input.js")) {
         return;
       }
 
